# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: host_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12host_manager.proto\"\x0e\n\x0cStopSparkMsg\"\x0f\n\rStartSparkMsg\"\x14\n\x12StopHostMonitorMsg\"Z\n\x13StartHostMonitorMsg\x12\x10\n\x08kafka_ip\x18\x01 \x01(\t\x12\x12\n\nkafka_port\x18\x02 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x04 \x01(\x05\"\x11\n\x0fStopFilebeatMsg\"\x12\n\x10StartFilebeatMsg\"\x97\x02\n\x11\x43onfigFilebeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x16\n\x0ereload_enabled\x18\x06 \x01(\x08\x12\x10\n\x08kafka_ip\x18\x07 \x01(\t\x12\x12\n\nkafka_port\x18\x08 \x01(\x05\x12\r\n\x05kafka\x18\t \x01(\x08\x12\x14\n\x0ckafka_topics\x18\n \x03(\t\x12\x18\n\x10\x66ilebeat_modules\x18\x0b \x03(\t\x12\x17\n\x0flog_files_paths\x18\x0c \x03(\t\"\x13\n\x11StopPacketbeatMsg\"\x14\n\x12StartPacketbeatMsg\"\x83\x01\n\x13\x43onfigPacketbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\"\x13\n\x11StopMetricbeatMsg\"\x14\n\x12StartMetricbeatMsg\"\xdd\x01\n\x13\x43onfigMetricbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x10\n\x08kafka_ip\x18\x06 \x01(\t\x12\x12\n\nkafka_port\x18\x07 \x01(\x05\x12\x16\n\x0ereload_enabled\x18\x08 \x01(\x08\x12\x1a\n\x12metricbeat_modules\x18\x0c \x03(\t\"\x12\n\x10GetHostStatusMsg\"\x95\x01\n\rHostStatusDTO\x12\x17\n\x0fmonitor_running\x18\x01 \x01(\x08\x12\x18\n\x10\x66ilebeat_running\x18\x02 \x01(\x08\x12\x1a\n\x12packetbeat_running\x18\x03 \x01(\x08\x12\x1a\n\x12metricbeat_running\x18\x04 \x01(\x08\x12\x19\n\x11heartbeat_running\x18\x05 \x01(\x08\"G\n\x11GetHostMetricsMsg\x12\x1b\n\x13\x66\x61iled_auth_last_ts\x18\x01 \x01(\x02\x12\x15\n\rlogin_last_ts\x18\x02 \x01(\x02\"\xd1\x01\n\x0eHostMetricsDTO\x12\x1b\n\x13num_logged_in_users\x18\x01 \x01(\x05\x12!\n\x19num_failed_login_attempts\x18\x02 \x01(\x05\x12\x1c\n\x14num_open_connections\x18\x03 \x01(\x05\x12\x18\n\x10num_login_events\x18\x04 \x01(\x05\x12\x15\n\rnum_processes\x18\x05 \x01(\x05\x12\x11\n\tnum_users\x18\x06 \x01(\x05\x12\n\n\x02ip\x18\x07 \x01(\t\x12\x11\n\ttimestamp\x18\x08 \x01(\x02\"\x12\n\x10StopHeartbeatMsg\"\x13\n\x11StartHeartbeatMsg\"\x9c\x01\n\x12\x43onfigHeartbeatMsg\x12\x11\n\tkibana_ip\x18\x01 \x01(\t\x12\x13\n\x0bkibana_port\x18\x02 \x01(\x05\x12\x12\n\nelastic_ip\x18\x03 \x01(\t\x12\x14\n\x0c\x65lastic_port\x18\x04 \x01(\x05\x12\x1a\n\x12num_elastic_shards\x18\x05 \x01(\x05\x12\x18\n\x10hosts_to_monitor\x18\x06 \x03(\t2\xf6\x07\n\x0bHostManager\x12\x38\n\x0fstopHostMonitor\x12\x13.StopHostMonitorMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10startHostMonitor\x12\x14.StartHostMonitorMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rgetHostStatus\x12\x11.GetHostStatusMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x37\n\x0egetHostMetrics\x12\x12.GetHostMetricsMsg\x1a\x0f.HostMetricsDTO\"\x00\x12\x32\n\x0cstopFilebeat\x12\x10.StopFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rstartFilebeat\x12\x11.StartFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0e\x63onfigFilebeat\x12\x12.ConfigFilebeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estopPacketbeat\x12\x12.StopPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0fstartPacketbeat\x12\x13.StartPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10\x63onfigPacketbeat\x12\x14.ConfigPacketbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estopMetricbeat\x12\x12.StopMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0fstartMetricbeat\x12\x13.StartMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12:\n\x10\x63onfigMetricbeat\x12\x14.ConfigMetricbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x34\n\rstopHeartbeat\x12\x11.StopHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x36\n\x0estartHeartbeat\x12\x12.StartHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12\x38\n\x0f\x63onfigHeartbeat\x12\x13.ConfigHeartbeatMsg\x1a\x0e.HostStatusDTO\"\x00\x12,\n\tstopSpark\x12\r.StopSparkMsg\x1a\x0e.HostStatusDTO\"\x00\x12.\n\nstartSpark\x12\x0e.StartSparkMsg\x1a\x0e.HostStatusDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'host_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPSPARKMSG._serialized_start=22
  _STOPSPARKMSG._serialized_end=36
  _STARTSPARKMSG._serialized_start=38
  _STARTSPARKMSG._serialized_end=53
  _STOPHOSTMONITORMSG._serialized_start=55
  _STOPHOSTMONITORMSG._serialized_end=75
  _STARTHOSTMONITORMSG._serialized_start=77
  _STARTHOSTMONITORMSG._serialized_end=167
  _STOPFILEBEATMSG._serialized_start=169
  _STOPFILEBEATMSG._serialized_end=186
  _STARTFILEBEATMSG._serialized_start=188
  _STARTFILEBEATMSG._serialized_end=206
  _CONFIGFILEBEATMSG._serialized_start=209
  _CONFIGFILEBEATMSG._serialized_end=488
  _STOPPACKETBEATMSG._serialized_start=490
  _STOPPACKETBEATMSG._serialized_end=509
  _STARTPACKETBEATMSG._serialized_start=511
  _STARTPACKETBEATMSG._serialized_end=531
  _CONFIGPACKETBEATMSG._serialized_start=534
  _CONFIGPACKETBEATMSG._serialized_end=665
  _STOPMETRICBEATMSG._serialized_start=667
  _STOPMETRICBEATMSG._serialized_end=686
  _STARTMETRICBEATMSG._serialized_start=688
  _STARTMETRICBEATMSG._serialized_end=708
  _CONFIGMETRICBEATMSG._serialized_start=711
  _CONFIGMETRICBEATMSG._serialized_end=932
  _GETHOSTSTATUSMSG._serialized_start=934
  _GETHOSTSTATUSMSG._serialized_end=952
  _HOSTSTATUSDTO._serialized_start=955
  _HOSTSTATUSDTO._serialized_end=1104
  _GETHOSTMETRICSMSG._serialized_start=1106
  _GETHOSTMETRICSMSG._serialized_end=1177
  _HOSTMETRICSDTO._serialized_start=1180
  _HOSTMETRICSDTO._serialized_end=1389
  _STOPHEARTBEATMSG._serialized_start=1391
  _STOPHEARTBEATMSG._serialized_end=1409
  _STARTHEARTBEATMSG._serialized_start=1411
  _STARTHEARTBEATMSG._serialized_end=1430
  _CONFIGHEARTBEATMSG._serialized_start=1433
  _CONFIGHEARTBEATMSG._serialized_end=1589
  _HOSTMANAGER._serialized_start=1592
  _HOSTMANAGER._serialized_end=2606
# @@protoc_insertion_point(module_scope)
