# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: docker_stats_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1a\x64ocker_stats_manager.proto\"P\n\x19StopDockerStatsMonitorMsg\x12\x11\n\temulation\x18\x01 \x01(\t\x12 \n\x18\x65xecution_first_ip_octet\x18\x02 \x01(\x05\",\n\x0b\x43ontainerIp\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x11\n\tcontainer\x18\x02 \x01(\t\"\xd5\x01\n\x1aStartDockerStatsMonitorMsg\x12\x11\n\temulation\x18\x01 \x01(\t\x12 \n\x18\x65xecution_first_ip_octet\x18\x02 \x01(\x05\x12\x10\n\x08kafka_ip\x18\x03 \x01(\t\x12\x1b\n\x13stats_queue_maxsize\x18\x04 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x05 \x01(\x05\x12\x12\n\nkafka_port\x18\x06 \x01(\x05\x12 \n\ncontainers\x18\x07 \x03(\x0b\x32\x0c.ContainerIp\" \n\x1eGetDockerStatsMonitorStatusMsg\"_\n\x15\x44ockerStatsMonitorDTO\x12\x14\n\x0cnum_monitors\x18\x01 \x01(\x05\x12\x12\n\nemulations\x18\x02 \x03(\t\x12\x1c\n\x14\x65mulation_executions\x18\x03 \x03(\x05\x32\x90\x02\n\x12\x44ockerStatsManager\x12X\n\x1bgetDockerStatsMonitorStatus\x12\x1f.GetDockerStatsMonitorStatusMsg\x1a\x16.DockerStatsMonitorDTO\"\x00\x12N\n\x16stopDockerStatsMonitor\x12\x1a.StopDockerStatsMonitorMsg\x1a\x16.DockerStatsMonitorDTO\"\x00\x12P\n\x17startDockerStatsMonitor\x12\x1b.StartDockerStatsMonitorMsg\x1a\x16.DockerStatsMonitorDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'docker_stats_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPDOCKERSTATSMONITORMSG._serialized_start=30
  _STOPDOCKERSTATSMONITORMSG._serialized_end=110
  _CONTAINERIP._serialized_start=112
  _CONTAINERIP._serialized_end=156
  _STARTDOCKERSTATSMONITORMSG._serialized_start=159
  _STARTDOCKERSTATSMONITORMSG._serialized_end=372
  _GETDOCKERSTATSMONITORSTATUSMSG._serialized_start=374
  _GETDOCKERSTATSMONITORSTATUSMSG._serialized_end=406
  _DOCKERSTATSMONITORDTO._serialized_start=408
  _DOCKERSTATSMONITORDTO._serialized_end=503
  _DOCKERSTATSMANAGER._serialized_start=506
  _DOCKERSTATSMANAGER._serialized_end=778
# @@protoc_insertion_point(module_scope)
